@@ -1,6 +1,6 @@
-adjCov <- function(object,id){
+adjCov <- function(object, id){
 
-# this does not work with power links and two-parameter copulas
+if(object$PL!="P") stop("This correction does not currently work for models with asymmetric links.")
 
 Vb     <- object$Vb 
 scores <- cbind( c(object$fit$dl.dbe1)*object$X1, c(object$fit$dl.dbe2)*object$X2, object$fit$dl.drho )
