@@ -1,4 +1,4 @@
-print.summary.SemiParBIVProbit <- function(x,digits = max(3, getOption("digits") - 3),
+print.summary.SemiParBIVProbit <- function(x, digits = max(3, getOption("digits") - 3),
                                              signif.stars = getOption("show.signif.stars"),...){
 
   if(x$BivD=="N")    cop <- "BIVARIATE NORMAL"
@@ -16,25 +16,8 @@ print.summary.SemiParBIVProbit <- function(x,digits = max(3, getOption("digits")
   if(x$BivD=="G90")  cop <- "BIVARIATE ROTATED GUMBEL COPULA (90 DEGREES)"
   if(x$BivD=="G180") cop <- "BIVARIATE SURVIVAL GUMBEL COPULA"
   if(x$BivD=="G270") cop <- "BIVARIATE ROTATED GUMBEL COPULA (270 DEGREES)"  
-    if(x$BivD=="BB1.0")   cop <- "BIVARIATE CLAYTON-GUMBEL COPULA" 
-    if(x$BivD=="BB1.90")  cop <- "BIVARIATE ROTATED CLAYTON-GUMBEL COPULA (90 DEGREES)"  
-    if(x$BivD=="BB1.180") cop <- "BIVARIATE ROTATED CLAYTON-GUMBEL COPULA (180 DEGREES)"  
-    if(x$BivD=="BB1.270") cop <- "BIVARIATE ROTATED CLAYTON-GUMBEL COPULA (270 DEGREES)"  
-    
-    if(x$BivD=="BB6.0")   cop <- "BIVARIATE JOE-GUMBEL COPULA" 
-    if(x$BivD=="BB6.90")  cop <- "BIVARIATE ROTATED JOE-GUMBEL COPULA (90 DEGREES)"  
-    if(x$BivD=="BB6.180") cop <- "BIVARIATE ROTATED JOE-GUMBEL COPULA (180 DEGREES)"  
-    if(x$BivD=="BB6.270") cop <- "BIVARIATE ROTATED JOE-GUMBEL COPULA (270 DEGREES)"  
-    
-    if(x$BivD=="BB7.0")   cop <- "BIVARIATE JOE-GUMBEL COPULA" 
-    if(x$BivD=="BB7.90")  cop <- "BIVARIATE ROTATED JOE-CLAYTON COPULA (90 DEGREES)"  
-    if(x$BivD=="BB7.180") cop <- "BIVARIATE ROTATED JOE-CLAYTON COPULA (180 DEGREES)"  
-    if(x$BivD=="BB7.270") cop <- "BIVARIATE ROTATED JOE-CLAYTON COPULA (270 DEGREES)"  
-    
-    if(x$BivD=="BB8.0")   cop <- "BIVARIATE JOE-FRANK COPULA" 
-    if(x$BivD=="BB8.90")  cop <- "BIVARIATE ROTATED JOE-FRANK COPULA (90 DEGREES)"  
-    if(x$BivD=="BB8.180") cop <- "BIVARIATE ROTATED JOE-FRANK COPULA (180 DEGREES)"  
-  if(x$BivD=="BB8.270") cop <- "BIVARIATE ROTATED JOE-FRANK COPULA (270 DEGREES)"  
+  
+  
   main.t <- "\nERRORS' DISTRIBUTION:" 
 
   cat(main.t,cop)
@@ -113,19 +96,11 @@ print.summary.SemiParBIVProbit <- function(x,digits = max(3, getOption("digits")
   if(x$BivD %in% c("N","T")) {cp <- "  rho = "; as.p <- x$rho} else{ cp <- "  theta = "; as.p <- x$theta}
 
 
-
-  del <- ""
-
-  if(x$BivD %in% c("BB1.0","BB1.180","BB1.90","BB1.270",
-                   "BB6.0","BB6.180","BB6.90","BB6.270",
-                   "BB7.0","BB7.180","BB7.90","BB7.270",
-                   "BB8.0","BB8.180","BB8.90","BB8.270") ) del <- paste("\ndelta = ",format(x$delta, digits=3),"(",format(x$CId[1],digits=3),",",format(x$CId[2],digits=3),")", sep="") 
-                   
-
-
-  if(x$sel==FALSE) cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")",del,"  Kendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","\ntotal edf = ",format(x$t.edf,digits=3),"  MR = ",format(x$MR,digits=3),"%","  QPS1 = ",format(x$QPS1,digits=3),"  QPS2 = ",format(x$QPS2,digits=3),"\nCR1 = ",format(x$CR1,digits=3),"%  CR2 = ",format(x$CR2,digits=3),"%\n\n", sep="")  
+  if(x$Model=="B" ) cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","  Kendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","\ntotal edf = ",format(x$t.edf,digits=3),"  MR = ",format(x$MR,digits=3),"%","  QPS1 = ",format(x$QPS1,digits=3),"  QPS2 = ",format(x$QPS2,digits=3),"\nCR1 = ",format(x$CR1,digits=3),"%  CR2 = ",format(x$CR2,digits=3),"%\n\n", sep="")  
      
-  if(x$sel==TRUE) cat("\nn = ",x$n,"  n.sel = ",x$n.sel,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")",del,"\nKendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
+  if(x$Model=="BSS") cat("\nn = ",x$n,"  n.sel = ",x$n.sel,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","\nKendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
+
+  if(x$Model=="BPO") cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","\nKendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
 
                        
 invisible(x)
